@@ -6,18 +6,17 @@ import numpy as np
 import psutil
 import torch
 import torch.optim as optim
-from torch.utils.tensorboard import SummaryWriter
 from tqdm import tqdm
 
-from ..scorer.ScorerTorch import ScorerTorch
-
 from ..DataSet import DataSet, TrainingSet
 from ..Model import Model
+from ..scorer.ScorerTorch import ScorerTorch
 
 
 class ModelTorchAdapter(Model):
     model: torch.nn.Module
     useGPU = torch.cuda.is_available()
+    useTensorBoard = False
 
     def _metrics(self, metric):
         return "val_" + metric["name"]
@@ -38,9 +37,9 @@ class ModelTorchAdapter(Model):
         x, y = datasetOrTuple
         
         if not torch.is_tensor(x):
-            x  = torch.tensor(x)
+            x = torch.tensor(x)
         if not torch.is_tensor(y):
-            y  = torch.tensor(y)
+            y = torch.tensor(y)
 
         dataset = DataSet(x, y)
 
@@ -63,23 +62,26 @@ class ModelTorchAdapter(Model):
         s.trace = True
         s.ignoreClasses = [self.ignoreClassIndex] if self.ignoreClassIndex is not None else []
 
-        validationData.transformToCategorical = False
-        validationData.continous = False
-        processList = tqdm(validationData, disable=(not self.showProgress))
-        processList.set_description("Validation")
+        validationData.transformToCategorical = 0
+        batchCount = len(validationData)
+        processList = tqdm(range(batchCount), disable=(not self.showProgress))
+        processList.set_description("Validation")       
+        batchGenerator = validationData.generator()
 
-        for record in processList:
-            x, y = self.prepareDataAdapter(record, validation=True)
+        for batchIndex in processList:
+            validationBatch = batchGenerator.__next__()
+            x, y = self.prepareDataAdapter(validationBatch, validation=True)
 
             # Run model
             output = model(x)
             batchPredictions = self.mapOutputForPrediction(output)
 
             if len(s.metrics) > 0:
-                y = y.reshape(-1)
                 if self.oneHotDecoded:
+                    y = y.reshape(-1, self.numClasses)
                     batchPredictions = batchPredictions.reshape(-1, self.numClasses)
                 else:
+                    y = y.reshape(-1)
                     batchPredictions = batchPredictions.reshape(-1)
 
                 results = s.score(y, batchPredictions, trace=True)
@@ -123,13 +125,19 @@ class ModelTorchAdapter(Model):
         return targets.float(), mask
 
     def train(self, dataset: TrainingSet):
-        self.summarywriter = SummaryWriter()
-
-        for (f, title) in self.startFigures:
-            self.summarywriter.add_figure(title, f)
+        
+        if ModelTorchAdapter.useTensorBoard:
+            from torch.utils.tensorboard import SummaryWriter
+            self.summarywriter = SummaryWriter()
+            
+            for (f, title) in self.startFigures:
+                self.summarywriter.add_figure(title, f)
+        else:
+            self.summarywriter = None
         self.cleanCsv()
 
-        self.oneHotDecoded = dataset.trainingData.transformToCategorical
+        if self.oneHotDecoded:
+            dataset.trainingData.transformToCategorical = self.numClasses
 
         metricDefinitions = {
             # init value, useAsBest, biggerIsBetter
@@ -181,10 +189,12 @@ class ModelTorchAdapter(Model):
             runningStats = {"loss": 0.0}
             batchesPerEpoch = len(dataset.trainingData)
             dataset.trainingData.continous = False
-            processList = tqdm(dataset.trainingData, disable=(not self.showProgress))
+            processList = tqdm(range(batchesPerEpoch), disable=(not self.showProgress))
             processList.set_description("EPOCH {}".format(i_epoch))
+            batchGenerator = dataset.trainingData.generator()
 
-            for trainBatch in processList:
+            for batchIndex in processList:
+                trainBatch = next(batchGenerator)
                 batchFeats, targs = self.prepareDataAdapter(trainBatch)
                 targs, mask = self.prepareTargetsForLoss(targs)
 
@@ -249,7 +259,9 @@ class ModelTorchAdapter(Model):
                 metricStrings.append(metricName + ": " + "{:.3f}".format(metricVal) + " [best: {:.3f}]".format(bestValue))
                 metricDiffStrings.append(metricName + ": " + "{:.3f}".format(diff))
                 metricValuetrings.append("{:.3f}".format(metricVal))
-                self.summarywriter.add_scalar(metricName, metricVal, global_step=self.fullEpochs)
+                
+                if self.summarywriter is not None:
+                    self.summarywriter.add_scalar(metricName, metricVal, global_step=self.fullEpochs)
 
                 isBigger = metricVal > bestValue
 
